#include <stdint.h>
#include <math.h>
#include "params.h"
#include "matrix.h"
#include "assert.h"

static double fmodq(double x)
{
  double c = x * PARAM_QINV;
  c = round(c);
  c *= PARAM_Q;
  return x-c;
}


void vector_add_n(double *r, const double *x, const double *y)
{
  unsigned int i;
  for(i=0;i<PARAM_N;i++)
    r[i] = fmodq(x[i] + y[i]);
}

void matrix_mul_mnk(double *r, const double *x, const double *y)
{
  unsigned int i,j,k;
  for(i=0;i<PARAM_M;i++)
  {
    for(j=0;j<PARAM_K;j++)
    {
      r[i*PARAM_K+j] = 0;
      for(k=0;k<PARAM_N;k++)
      {
        r[i*PARAM_K+j] += fmodq(x[i*PARAM_N+k] * y[k*PARAM_K+j]);
        r[i*PARAM_K+j]  = fmodq(r[i*PARAM_K+j]);
      }
    }
  }
}

void matrix_add_mk(double *r, const double *x, const double *y)
{
  unsigned int i;
  for(i=0;i<PARAM_M*PARAM_K;i++)
    r[i] = fmodq(x[i] + y[i]);
}

void mul_mk_gauss_m_sparse_subred(double res[PARAM_M], const unsigned char *pke, const uint32_t pos_list[PARAM_W], const double sign_list[PARAM_W])
{
  int i,j;
  int32_t val0;
  int32_t *ipk = (int32_t *)pke;
 
  for(j=0; j<PARAM_W; j++)
  {
    for(i=0; i<PARAM_M; i++)
    {
      val0 = ipk[PARAM_M*pos_list[j]+i+0];
      res[i+0] -= (double)val0*sign_list[j];
      res[i+0] = fmodq(res[i+0]);
    }
  }
}


void mul_mn_uniQ_n_uniB(double res[PARAM_M], const double b[PARAM_N], const int32_t a[PARAM_N][PARAM_M])
{
  int i,j;

  for(i=0; i<PARAM_M; i++)
  {
    res[i] = 0; 	
    for(j=0; j<PARAM_N; j++)
    { 
      res[i] += (double)a[j][i]*b[j];
      res[i] = fmodq(res[i]);
    }
  } 
}
